const path = require('path')
const fs = require('fs')
const minimist = require('minimist')
const ora = require('ora')
const CWD = process.cwd()
const args = minimist(process.argv.slice(2))
const distPath = path.resolve(__dirname, '../dist')
const apiOrigin = args.o || args.origin || ('http://localhost:5000')
const cpList = []

const es = ['SIGINT']

es.forEach(e => {
  process.on(e, function () {
    exitProcess(e)
  })
})

function exitProcess (s) {
  // console.clear()
  cpList.forEach(cp => cp.kill(s))
  process.exit(s)
}

function build () {
  return new Promise(resolve => {
    const spawn = require('child_process').spawn
    const spin = ora('processing...').start()
    const cp = spawn(/^win/.test(process.platform) ? 'vue-cli-service.cmd' : 'vue-cli-service', ['build'], { cwd: path.resolve(__dirname, '../') })

    cp.on('exit', function (err) {
      spin.stop()
      if (!err || err === 0) console.clear()
      resolve()
    })

    cp.stdout.pipe(process.stdout)

    cpList.push(cp)
  })
}

function isExistDistDir () {
  return new Promise((resolve, reject) => {
    fs.access(distPath, err => {
      resolve(!err)
    })
  })
}

async function beginServer () {
  const express = require('express')
  const { createProxyMiddleware } = require('http-proxy-middleware')
  const app = express()

  app.use(express.static(path.resolve(CWD, './dist')))

  app.use(
    createProxyMiddleware('/api', {
      target: apiOrigin,
      changeOrigin: true,
      pathRewrite: {
        '^/api': ''
      }
    })
  )

  try {
    const listenPort = await require('portfinder').getPortPromise({ port: 7000 })
    app.listen(listenPort, () => {
      console.clear()
      console.log('Page url: ', 'http://localhost:' + listenPort)
      console.log('Api: ', apiOrigin)
    })
  } catch (err) {
    exitProcess(err)
  }
}

(async function () {
  console.clear()
  const isExist = await isExistDistDir()

  if (isExist) {
    beginServer()
  } else {
    await build()

    const exist = await isExistDistDir()

    if (exist) beginServer()
  }
})()
